var express    = require('express')
var mongoose   = require('mongoose')
var passport   = require('passport')
var jwt        = require('jsonwebtoken')
var ExtractJwt = require('passport-jwt').ExtractJwt
var User       = require('../model/user')
var cors       = require('cors')

function resourceModel(resource) {
  var fields = {timestamp: { type: Date, default: Date.now }}
  resource.fields.map(field => {
    fields[field.name] = { type: field.type }
    if (field.required) {
      fields[field.name].required = [field.required, field.name + " field is required" ]
    }
  })
  var schema = new mongoose.Schema(fields)
  return mongoose.model(resource.name, schema)
}

function authenticate() {
  return passport.authenticate('jwt', { session: false })
}

function filterFields(resource, authorized) {
  var fields = ["timestamp"]
  resource.fields.forEach(field => {
    if (authorized || !field.private) {
      fields.push(field.name)
    }
  })
  return fields.join(" ")
}

function mountOperation(op, resource, router, callback) {
  if (resource.private) {
    // Mount it as a private resource
    return router[op]("/", cors(), authenticate(), callback)
  }

  return router[op]("/", cors(), callback)
}

function sendResponse(resource, res, err, data) {
  if(err) {
    return res.send({status: "fail", error: err});
  }
  return res.send({status: "success", [resource.name]: data})
}

function isRequestAuthorized(req, props, done) {
  var apiKey    = props.secure.apiKey
  var apiSecret = props.secure.apiSecret
  var jwtKey    = apiKey + ":" + apiSecret

  if (!req.headers.authorization) {
    return done(false)
  }

  jwt.verify(req.headers.authorization.substring(4), jwtKey, (err, payload) => {
    if (err) {
      return done(false)
    }
    User.findById(payload._doc._id, (err, user) => {
      if (err) {
        return done(false)
      }

      if (!user) {
        return done(false)
      }

      return done(true)
    })
  })
}

function mountRoute(props, resource) {
  var router = express.Router()
  var model  = resourceModel(resource)

  // Add the retrieval operation
  mountOperation('get', resource, router, function(req, res) {
    isRequestAuthorized(req, props, function(authorized) {
      var fields = filterFields(resource, authorized)
      model.find({}, fields, function(err, data) {
        return sendResponse(resource, res, err, data)
      })
    })
  })

  // Add the creation operation
  mountOperation('post', resource, router, function(req, res) {
    model.create(req.body.data, function(err, data) {
      return sendResponse(resource, res, err, data)
    })
  })

  return router
}

function mountRoutes(app, resources, props) {
  resources.forEach(resource => app.use("/" + resource.name, mountRoute(props, resource)))
}

module.exports = mountRoutes
