var express       = require('express')
var passport      = require('passport')
var User          = require('../model/user')
var jwt           = require('jsonwebtoken')
var JwtStrategy   = require('passport-jwt').Strategy
var ExtractJwt    = require('passport-jwt').ExtractJwt
var LocalStrategy = require('passport-local')
var cors          = require('cors')

function addAuthentication(jwtKey, router) {
    var localLogin = new LocalStrategy({
      usernameField: 'email'
    }, (email, password, done) => {
      User.findOne({ email: email }, (err, user) => {
        if (err) {
          return done(err)
        }

        if (!user) {
          return done(null, false, {error: 'Unauthorized'})
        }

        if (!user.passwordMatch(password)) {
          return done(null, false, {error: 'bad password'})
        }

        return done(null, user)
      })
    })

    var jwtLogin = new JwtStrategy({
      jwtFromRequest: ExtractJwt.fromAuthHeader(),
      secretOrKey: jwtKey
    }, (payload, done) => {
      User.findById(payload._doc._id, (err, user) => {
        if (err) {
          return done(err)
        }

        if (!user) {
          return done(null, false, {error: 'Unauthorized'})
        }

        return done(null, user)
      })
    })

    passport.use(localLogin)
    passport.use(jwtLogin)
}

function addRegistration(router) {
  router.post('/register', cors(), (req, res) => {
    if (!req.body.email) {
      return res.json({
        status: "fail",
        message: 'Please enter your email'
      })
    }

    if (!req.body.password) {
      return res.json({
        status: "fail",
        message: 'Please enter your password'
      })
    }

    var newUser = new User({
      email: req.body.email,
      password: req.body.password
    })

    // Make sure the password gets hashed before saving
    newUser.hashPassword()

    newUser.save((error) => {
      if (error) {
        return res.json({
          status: "fail",
          error: 'The email already exists'
        })
      }

      return res.json({
        status: "success",
        message: "Successfully created new user"
      })
    })
  })
}

function addLogin(jwtKey, router) {
  router.post('/login', cors(), (req, res) => {
    User.findOne({ email: req.body.email }, (error, user) => {
        if (error) {
          return res.send({status: "fail", error: error})
        }

        if (!user) {
          return res.send({status: "fail", error: 'Could not find user'})
        }

        // The user was found, let's check the password
        if (!user.passwordMatch(req.body.password)) {
          return res.send({status: "fail", error: 'Invalid password'})
        }

        var token = jwt.sign(user, jwtKey, {
          expiresIn: "10 days"
        })

        res.send({
          status: "success",
          message: "Successful authentication",
          token: token
        })
    })
  })
}

function mountRoute(app, props) {
  var router = express.Router()

  // Look for the api key
  var apiKey    = props.secure.apiKey
  var apiSecret = props.secure.apiSecret
  var jwtKey    = apiKey + ":" + apiSecret

  addAuthentication(jwtKey, router)
  addRegistration(router)
  addLogin(jwtKey, router)

  app.use('/auth', router)
}

module.exports = mountRoute
